program aqr;

(************************************************************************
 *                                                                      *
 *   Ant QuickRes                                                       *
 *   (C) 2002-2003 Antoine Potten                                       *
 *   software@antp.be - http://www.antp.be/software                     *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 2     *
 *   of the License, or (at your option) any later version.             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 ************************************************************************)

{$APPTYPE CONSOLE}

uses
  SysUtils, Windows;

function ChangeDisplaySettings2(lpDevMode: Pointer; dwFlags: DWORD): Longint; stdcall; external user32 name 'ChangeDisplaySettingsA';
// redefined since ChangeDisplaySettings requires a "var", and to use the "restore previous mode" we need to pass a "nil" parameter

var
  DevMode: TDeviceMode;
  Flags: Cardinal;
  Result: LongInt;
  Switches: record
    Help: Boolean;          // -f
    Width: Boolean;         // -wx
    Height: Boolean;        // -hx
    Colors: Boolean;        // -cx
    Frequency: Boolean;     // -fx
    TestOnly: Boolean;      // -t
    MakePermanent: Boolean; // -p
    Restore: Boolean;       // -r
  end;
  Values: record
    Width: DWORD;
    Height: DWORD;
    Colors: DWORD;
    Frequency: DWORD;
  end;
  i, L: Integer;
  aParam: string;

  ExitStatus: string;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

begin
  Writeln('Ant QuickRes 1.1');
  Writeln('(C) 2002-2003 Antoine Potten - www.antp.be');
  Writeln('Command-line tool to modify screen resolution');
  Writeln('');
  try
    with Switches do
    begin
      Help := False;
      Width := False;
      Height := False;
      Colors := False;
      Frequency := False;
      TestOnly := False;
      MakePermanent := False;
      Restore := False;
    end;
    try
      for i := 1 to ParamCount do
      begin
        aParam := ParamStr(i);
        L := Length(aParam);
        if(L > 1) then
        begin
          if aParam[1] <> '-' then
          begin
            Switches.Help := True;
            Break;
          end else
            case aParam[2] of
              'w':
                begin
                  Switches.Width := True;
                  Values.Width := StrToInt(Copy(aParam, 3, L));
                end;
              'h':
                begin
                  Switches.Height := True;
                  Values.Height := StrToInt(Copy(aParam, 3, L));
                end;
              'c':
                begin
                  Switches.Colors := True;
                  Values.Colors := StrToInt(Copy(aParam, 3, L));
                end;
              'f':
                begin
                  Switches.Frequency := True;
                  Values.Frequency := StrToInt(Copy(aParam, 3, L));
                end;
              't':
                begin
                  Switches.TestOnly := True;
                end;
              'p':
                begin
                  Switches.MakePermanent := True;
                end;
              'r':
                begin
                  Switches.Restore := True;
                end;
            else
              Switches.Help := True;
              Break;
            end;
        end;
      end;
    except
      Switches.Help := True;
    end;
    if (ParamCount = 0) or (Switches.Help) then
    begin
      Writeln('Usage:');
      Writeln('aqr.exe [-wx] [-hx] [-cx] [-fx] [-t] [-p] [-r]');
      Writeln('');
      Writeln('Change display settings:');
      Writeln('-w = Specify a new value for horizontal resolution (pixels)');
      Writeln('-h = Specify a new value for vertical resolution (pixels)');
      Writeln('-c = Specify a new color depth (number of bits per pixel)');
      Writeln('-f = Specify a new vertical refresh rate (hertz)');
      Writeln('');
      Writeln('Action:');
      Writeln('-t = Only test new mode');
      Writeln('-p = Make change permanent (store new mode in the registry');
      Writeln('-r = Restore previous mode (last mode stored in the registry');
      Writeln('');
      Writeln('Notes:');
      Writeln('Switches t and p are not compatible together so they should');
      Writeln('not be used at the same time.');
      Writeln('Any bad switch or value will display this help');
      Writeln('');
      Writeln('Sample command:');
      Writeln('aqr.exe -w800 -h600 -c32 -f85 -p');
    end else
    begin
      FillChar(DevMode, SizeOf(DevMode), #0);
      DevMode.dmSize := SizeOf(DevMode);
      if Switches.Width then
      begin
        DevMode.dmPelsWidth := Values.Width;
        DevMode.dmFields := DevMode.dmFields or DM_PELSWIDTH;
      end;
      if Switches.Height then
      begin
        DevMode.dmPelsHeight := Values.Height;
        DevMode.dmFields := DevMode.dmFields or DM_PELSHEIGHT;
      end;
      if Switches.Colors then
      begin
        DevMode.dmBitsPerPel := Values.Colors;
        DevMode.dmFields := DevMode.dmFields or DM_BITSPERPEL;
      end;
      if Switches.Frequency then
      begin
        DevMode.dmDisplayFrequency := Values.Frequency;
        DevMode.dmFields := DevMode.dmFields or DM_DISPLAYFREQUENCY;
      end;
      if Switches.TestOnly then
      begin
        Flags := CDS_TEST;
      end else
      if Switches.MakePermanent then
      begin
        Flags := CDS_UPDATEREGISTRY;
      end else
        Flags := 0;
      if Switches.Restore then
        Result := ChangeDisplaySettings2(nil, 0)
      else
        Result := ChangeDisplaySettings2(@DevMode, Flags);
      case Result of
        DISP_CHANGE_SUCCESSFUL: ExitStatus := 'The settings change was successful.';
        DISP_CHANGE_RESTART:    ExitStatus := 'The computer must be restarted in order for the graphics mode to work.';
        DISP_CHANGE_BADFLAGS:   ExitStatus := 'An invalid set of flags was passed in.';
        DISP_CHANGE_FAILED:     ExitStatus := 'The display driver failed the specified graphics mode.';
        DISP_CHANGE_BADMODE:    ExitStatus := 'The graphics mode is not supported.';
        DISP_CHANGE_NOTUPDATED: ExitStatus := 'Unable to write settings to the registry.';
      else
        raise Exception.Create(Format('Unexpected result (%d) from ChangeDisplaySettings()', [Result]));
      end;
      Writeln('Exit Status: ' + ExitStatus);
    end;
  except
    on e: Exception do
      Writeln('Exit with Error: ' + e.Message);
  end;
end.
 
